@@ -5,9 +5,9 @@
 *
 * @package   awl
 * @subpackage   Utilities
-* @author    Andrew McMillan <andrew@catalyst.net.nz>
-* @copyright Catalyst IT Ltd
-* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
+* @author    Andrew McMillan <andrew@mcmillan.net.nz>
+* @copyright Catalyst IT Ltd, Morphoss Ltd <http://www.morphoss.com/>
+* @license   http://www.gnu.org/licenses/lgpl-3.0.txt  GNU LGPL version 3 or later
 */
 
 if ( !function_exists('dbg_error_log') ) {
@@ -26,7 +26,6 @@ if ( !function_exists('dbg_error_log') ) {
   */
   function dbg_error_log() {
     global $c;
-    $argc = func_num_args();
     $args = func_get_args();
     $type = "DBG";
     $component = array_shift($args);
@@ -43,6 +42,7 @@ if ( !function_exists('dbg_error_log') ) {
     }
     else if ( !isset($c->dbg[strtolower($component)]) ) return;
 
+    $argc = func_num_args();
     if ( 2 <= $argc ) {
       $format = array_shift($args);
     }
@@ -57,12 +57,20 @@ if ( !function_exists('dbg_error_log') ) {
 
 if ( !function_exists('apache_request_headers') ) {
   /**
-  * Forward compatibility so we can use the non-deprecated name in PHP4
+  * Compatibility so we can use the apache function name and still work with CGI
   * @package awl
   */
-  function apache_request_headers() {
-    return getallheaders();
-  }
+  eval('
+    function apache_request_headers() {
+        foreach($_SERVER as $key=>$value) {
+            if (substr($key,0,5)=="HTTP_") {
+                $key=str_replace(" ","-",ucwords(strtolower(str_replace("_"," ",substr($key,5)))));
+                $out[$key]=$value;
+            }
+        }
+        return $out;
+    }
+  ');
 }
 
 
@@ -93,6 +101,21 @@ if ( !function_exists('dbg_log_array') ) {
 
 
 
+if ( !function_exists("session_simple_md5") ) {
+  /**
+  * Make a plain MD5 hash of a string, identifying the type of hash it is
+  *
+  * @param string $instr The string to be salted and MD5'd
+  * @return string The *MD5* and the MD5 of the string
+  */
+  function session_simple_md5( $instr ) {
+    dbg_error_log( "Login", "Making plain MD5: instr=$instr, md5($instr)=".md5($instr) );
+    return ( '*MD5*'. md5($instr) );
+  }
+}
+
+
+
 if ( !function_exists("session_salted_md5") ) {
   /**
   * Make a salted MD5 string, given a string and (possibly) a salt.
@@ -112,7 +135,7 @@ if ( !function_exists("session_salted_md5") ) {
 
 
 
-if ( !function_exists("session_salted_sha1") && version_compare(phpversion(), "4.9.9") > 0 ) {
+if ( !function_exists("session_salted_sha1") ) {
   /**
   * Make a salted SHA1 string, given a string and (possibly) a salt.  PHP5 only (although it
   * could be made to work on PHP4 (@see http://www.openldap.org/faq/data/cache/347.html). The
@@ -175,7 +198,12 @@ EOERRMSG;
       }
     }
 
-    if ( preg_match('/^\*(.+)\*.+$/', $we_have, $regs ) ) {
+    if ( preg_match('/^\*MD5\*.+$/', $we_have, $regs ) ) {
+      // A crappy unsalted md5sum like "*MD5*<md5>"
+      $md5_sent = session_simple_md5( $they_sent ) ;
+      return ( $md5_sent == $we_have );
+    }
+    else if ( preg_match('/^\*(.+)\*.+$/', $we_have, $regs ) ) {
       // A nicely salted md5sum like "*<salt>*<salted_md5>"
       $salt = $regs[1];
       $md5_sent = session_salted_md5( $they_sent, $salt ) ;
@@ -269,7 +297,7 @@ if ( !function_exists("uuid") ) {
 }
 
 if ( !function_exists("translate") ) {
-  require_once("Translation.php");
+  require("Translation.php");
 }
 
  if ( !function_exists("clone") && version_compare(phpversion(), '5.0') < 0) {
@@ -385,7 +413,7 @@ if ( !function_exists("get_fields") ) {
 
     if ( !isset($_AWL_field_cache[$tablename]) ) {
       dbg_error_log( "DataUpdate", ":get_fields: Loaded fields for table '$tablename'" );
-      $sql = "SELECT f.attname, t.typname FROM pg_attribute f ";
+      $sql = "SELECT f.attname, t.typname, f.atttypmod FROM pg_attribute f ";
       $sql .= "JOIN pg_class c ON ( f.attrelid = c.oid ) ";
       $sql .= "JOIN pg_type t ON ( f.atttypid = t.oid ) ";
       $sql .= "WHERE relname = ? AND attnum >= 0 order by f.attnum;";
@@ -393,7 +421,7 @@ if ( !function_exists("get_fields") ) {
       $qry->Exec("DataUpdate");
       $fields = array();
       while( $row = $qry->Fetch() ) {
-        $fields["$row->attname"] = $row->typname;
+        $fields["$row->attname"] = $row->typname . ($row->atttypmod != -1 ? sprintf('(%d)',$row->atttypmod) : '');
       }
       $_AWL_field_cache[$tablename] = $fields;
     }
@@ -402,7 +430,7 @@ if ( !function_exists("get_fields") ) {
 }
 
 
-if ( !function_exists("force_utf8") && isset($need_force_utf8) && $need_force_utf8 ) {
+if ( !function_exists("force_utf8") ) {
   function define_byte_mappings() {
     global $byte_map, $nibble_good_chars;
 
@@ -482,3 +510,15 @@ if ( !function_exists("force_utf8") && isset($need_force_utf8) && $need_force_ut
   }
 
 }
+
+
+/**
+* Try and extract something like "Pacific/Auckland" or "America/Indiana/Indianapolis" if possible.
+*/
+function olson_from_tzstring( $tzstring ) {
+  if ( preg_match( '{((Antarctica|America|Africa|Atlantic|Asia|Indian|Europe|Pacific)/(([^/]+)/)?[^/]+)$}', $tzstring, $matches ) ) {
+//    dbg_error_log( 'INFO', 'Found timezone "%s" from string "%s"', $matches[1], $tzstring );
+    return $matches[1];
+  }
+  return null;
+}
