@@ -46,7 +46,8 @@
 * @license   http://gnu.org/copyleft/gpl.html GNU GPL v2 or later
 *
 */
-require_once("XMLElement.php");
+require_once('XMLElement.php');
+require_once('PgQuery.php');
 
 /**
 * A Class for representing properties within an iCalendar
@@ -133,7 +134,7 @@ class iCalProp {
       $value = substr( $v, $pos + 1);
       $this->parameters[$name] = $value;
     }
-    dbg_error_log("iCalendar", " iCalProp::ParseFrom found '%s' = '%s' with %d parameters", $this->name, substr($this->content,0,200), count($this->parameters) );
+//    dbg_error_log('iCalendar', " iCalProp::ParseFrom found '%s' = '%s' with %d parameters", $this->name, substr($this->content,0,200), count($this->parameters) );
   }
 
 
@@ -148,7 +149,7 @@ class iCalProp {
     if ( $newname != null ) {
       $this->name = $newname;
       if ( isset($this->rendered) ) unset($this->rendered);
-      dbg_error_log("iCalendar", " iCalProp::Name(%s)", $this->name );
+//      dbg_error_log('iCalendar', " iCalProp::Name(%s)", $this->name );
     }
     return $this->name;
   }
@@ -370,7 +371,7 @@ class iCalComponent {
     foreach( $this->properties AS $k => $v ) {
       $also = $v->GetParameterValue($parameter_name);
       if ( isset($also) && $also != "" ) {
-        dbg_error_log( "iCalendar", "::CollectParameterValues(%s) : Found '%s'", $parameter_name, $also);
+//        dbg_error_log( 'iCalendar', "::CollectParameterValues(%s) : Found '%s'", $parameter_name, $also);
         $values[$also] = 1;
       }
     }
@@ -408,7 +409,7 @@ class iCalComponent {
       }
       if ( preg_match('/^\s*$/', $line ) ) continue;
       $line = rtrim( $line, "\r\n" );
-      dbg_error_log( "iCalendar",  "::ParseFrom: Parsing line: $line");
+//      dbg_error_log( 'iCalendar',  "::ParseFrom: Parsing line: $line");
 
       if ( $type === false ) {
         if ( preg_match( '/^BEGIN:(.+)$/', $line, $matches ) ) {
@@ -416,21 +417,20 @@ class iCalComponent {
           $type = $matches[1];
           $finish = "END:$type";
           $this->type = $type;
-          dbg_error_log( "iCalendar", "::ParseFrom: Start component of type '%s'", $type);
+          dbg_error_log( 'iCalendar', "::ParseFrom: Start component of type '%s'", $type);
         }
         else {
-          dbg_error_log( "iCalendar", "::ParseFrom: Ignoring crap before start of component: $line");
+          dbg_error_log( 'iCalendar', "::ParseFrom: Ignoring crap before start of component: $line");
           // unset($lines[$k]);  // The content has crap before the start
           if ( $line != "" ) $this->rendered = null;
         }
       }
       else if ( $type == null ) {
-        dbg_error_log( "iCalendar", "::ParseFrom: Ignoring crap after end of component");
-        unset($lines[$k]);  // The content has crap after the end
+        dbg_error_log( 'iCalendar', "::ParseFrom: Ignoring crap after end of component");
         if ( $line != "" ) $this->rendered = null;
       }
       else if ( $line == $finish ) {
-        dbg_error_log( "iCalendar", "::ParseFrom: End of component");
+        dbg_error_log( 'iCalendar', "::ParseFrom: End of component");
         $type = null;  // We have reached the end of our component
       }
       else {
@@ -439,22 +439,22 @@ class iCalComponent {
           $subtype = $matches[1];
           $subfinish = "END:$subtype";
           $subcomponent = $line . "\r\n";
-          dbg_error_log( "iCalendar", "::ParseFrom: Found a subcomponent '%s'", $subtype);
+          dbg_error_log( 'iCalendar', "::ParseFrom: Found a subcomponent '%s'", $subtype);
         }
         else if ( $subtype ) {
           // We are inside a sub-component
           $subcomponent .= $this->WrapComponent($line);
           if ( $line == $subfinish ) {
-            dbg_error_log( "iCalendar", "::ParseFrom: End of subcomponent '%s'", $subtype);
+            dbg_error_log( 'iCalendar', "::ParseFrom: End of subcomponent '%s'", $subtype);
             // We have found the end of a sub-component
             $this->components[] = new iCalComponent($subcomponent);
             $subtype = false;
           }
-          else
-            dbg_error_log( "iCalendar", "::ParseFrom: Inside a subcomponent '%s'", $subtype );
+//          else
+//            dbg_error_log( 'iCalendar', "::ParseFrom: Inside a subcomponent '%s'", $subtype );
         }
         else {
-          dbg_error_log( "iCalendar", "::ParseFrom: Parse property of component");
+//          dbg_error_log( 'iCalendar', "::ParseFrom: Parse property of component");
           // It must be a normal property line within a component.
           $this->properties[] = new iCalProp($line);
         }
@@ -600,7 +600,7 @@ class iCalComponent {
       $new_prop->Name($new_property);
       $new_prop->Value($value);
       if ( $parameters != null ) $new_prop->Parameters($parameters);
-      dbg_error_log("iCalendar"," Adding new property '%s'", $new_prop->Render() );
+      dbg_error_log('iCalendar'," Adding new property '%s'", $new_prop->Render() );
       $this->properties[] = $new_prop;
     }
     else if ( gettype($new_property) ) {
@@ -836,20 +836,20 @@ class iCalComponent {
   */
   function GetPropertiesByPath( $path ) {
     $properties = array();
-    dbg_error_log( "iCalendar", "GetPropertiesByPath: Querying within '%s' for path '%s'", $this->type, $path );
+    dbg_error_log( 'iCalendar', "GetPropertiesByPath: Querying within '%s' for path '%s'", $this->type, $path );
     if ( !preg_match( '#(/?)(!?)([^/]+)(/?.*)$#', $path, $matches ) ) return $properties;
 
     $adrift = ($matches[1] == '');
     $normal = ($matches[2] == '');
     $ourtest = $matches[3];
     $therest = $matches[4];
-    dbg_error_log( "iCalendar", "GetPropertiesByPath: Matches: %s -- %s -- %s -- %s\n", $matches[1], $matches[2], $matches[3], $matches[4] );
+    dbg_error_log( 'iCalendar', "GetPropertiesByPath: Matches: %s -- %s -- %s -- %s\n", $matches[1], $matches[2], $matches[3], $matches[4] );
     if ( $ourtest == '*' || (($ourtest == $this->type) === $normal) && $therest != '' ) {
       if ( preg_match( '#^/(!?)([^/]+)$#', $therest, $matches ) ) {
         $normmatch = ($matches[1] =='');
         $proptest  = $matches[2];
         foreach( $this->properties AS $k => $v ) {
-          if ( $proptest = '*' || (($v->Name() == $proptest) === $normmatch ) ) {
+          if ( $proptest == '*' || (($v->Name() == $proptest) === $normmatch ) ) {
             $properties[] = $v;
           }
         }
@@ -872,7 +872,7 @@ class iCalComponent {
         $properties = array_merge( $properties, $v->GetPropertiesByPath($path) );
       }
     }
-    dbg_error_log("iCalendar", "GetPropertiesByPath: Found %d within '%s' for path '%s'\n", count($properties), $this->type, $path );
+    dbg_error_log('iCalendar', "GetPropertiesByPath: Found %d within '%s' for path '%s'\n", count($properties), $this->type, $path );
     return $properties;
   }
 
@@ -880,17 +880,30 @@ class iCalComponent {
 
 /**
 ************************************************************************************
-* Pretty much everything below here is deprecated and should be avoided in favour
+* Everything below here is deprecated and should be avoided in favour
 * of using, improving and enhancing the more sensible structures above.
 ************************************************************************************
 */
 
+function deprecated( $method ) {
+  global $c;
+  if ( isset($c->dbg['ALL']) || isset($c->dbg['deprecated'])  || isset($c->dbg['icalendar']) ) {
+    $stack = debug_backtrace();
+    array_shift($stack);
+    if ( preg_match( '{/inc/iCalendar.php$}', $stack[0]['file'] ) && $stack[0]['line'] > __LINE__ ) return;
+    dbg_error_log("LOG", " iCalendar: Call to deprecated method '%s'", $method );
+    foreach( $stack AS $k => $v ) {
+      dbg_error_log( 'LOG', ' iCalendar: Deprecated call from line %4d of %s', $v['line'], $v['file']);
+    }
+  }
+}
+
 /**
-* A Class for handling Events on a calendar
+* A Class for handling Events on a calendar (DEPRECATED)
 *
 * @package awl
 */
-class iCalendar {
+class iCalendar {  // DEPRECATED
   /**#@+
   * @access private
   */
@@ -928,6 +941,7 @@ class iCalendar {
   /**#@-*/
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * The constructor takes an array of args.  If there is an element called 'icalendar'
   * then that will be parsed into the iCalendar object.  Otherwise the array elements
   * are converted into properties of the iCalendar object directly.
@@ -935,6 +949,7 @@ class iCalendar {
   function iCalendar( $args ) {
     global $c;
 
+    deprecated('iCalendar::iCalendar');
     $this->tz_locn = "";
     if ( !isset($args) || !(is_array($args) || is_object($args)) ) return;
     if ( is_object($args) ) {
@@ -978,7 +993,7 @@ class iCalendar {
     $this->properties = array();
 
     foreach( $args AS $k => $v ) {
-      dbg_error_log( "iCalendar", ":Initialise: %s to >>>%s<<<", $k, $v );
+      dbg_error_log( 'iCalendar', ":Initialise: %s to >>>%s<<<", $k, $v );
       $property = new iCalProp();
       $property->Name($k);
       $property->Value($v);
@@ -1002,14 +1017,16 @@ class iCalendar {
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Save any timezones by TZID in the PostgreSQL database for future re-use.
   */
   function SaveTimeZones() {
     global $c;
 
+    deprecated('iCalendar::SaveTimeZones');
     $this->tzid_list = array_keys($this->component->CollectParameterValues('TZID'));
     if ( ! isset($this->tzid) && count($this->tzid_list) > 0 ) {
-      dbg_error_log( "icalendar", "::TZID_List[0] = '%s', count=%d", $this->tzid_list[0], count($this->tzid_list) );
+      dbg_error_log( 'iCalendar', "::TZID_List[0] = '%s', count=%d", $this->tzid_list[0], count($this->tzid_list) );
       $this->tzid = $this->tzid_list[0];
     }
 
@@ -1023,25 +1040,20 @@ class iCalendar {
         $tzid = $tz->GetPValue('TZID');
 
         $qry = new PgQuery( "SELECT tz_locn FROM time_zone WHERE tz_id = ?;", $tzid );
-        if ( $qry->Exec('iCalendar') && $qry->rows == 1 ) {
+        if ( $qry->Exec('iCalendar') && $qry->rows() == 1 ) {
           $row = $qry->Fetch();
           if ( !isset($first_tzid) ) $first_tzid = $row->tz_locn;
           continue;
         }
 
-        if ( $tzid != "" && $qry->rows == 0 ) {
+        if ( $tzid != "" && $qry->rows() == 0 ) {
 
           $tzname = $tz->GetPValue('X-LIC-LOCATION');
-          if ( !isset($tzname) ) {
-            /**
-            * Try and convert the TZID to a string like "Pacific/Auckland" if possible.
-            */
-            $tzname = preg_replace('#^(.*[^a-z])?([a-z]+/[a-z]+)$#i','$2',$tzid );
-          }
+          if ( !isset($tzname) ) $tzname = olson_from_tzstring($tzid);
 
           $qry2 = new PgQuery( "INSERT INTO time_zone (tz_id, tz_locn, tz_spec) VALUES( ?, ?, ? );",
                                       $tzid, $tzname, $tz->Render() );
-          $qry2->Exec("iCalendar");
+          $qry2->Exec('iCalendar');
         }
       }
     }
@@ -1052,7 +1064,7 @@ class iCalendar {
       if ( preg_match( '#\S+/\S+#', $tzname) ) {
         $this->tz_locn = $tzname;
       }
-      dbg_error_log( "icalendar", " TZCrap1: TZID '%s', Location '%s', Perhaps: %s", $tzid, $this->tz_locn, $tzname );
+      dbg_error_log( 'iCalendar', " TZCrap1: TZID '%s', Location '%s', Perhaps: %s", $tzid, $this->tz_locn, $tzname );
     }
 
     if ( (!isset($this->tz_locn) || $this->tz_locn == "") && isset($c->local_tzid) ) {
@@ -1065,7 +1077,7 @@ class iCalendar {
   /**
   * An array of property names that we should always want when rendering an iCalendar
   *
-  * @deprecated This function is deprecated and will be removed eventually.
+  * @DEPRECATED: This class will be removed soon.
   * @todo Remove this function.
   */
   function DefaultPropertyList() {
@@ -1084,16 +1096,16 @@ class iCalendar {
   *
   * @return string A string from BEGIN:SOMETHING to END:SOMETHING, possibly multiple of these
   *
-  * @deprecated This function is deprecated and will be removed eventually.
+  * @DEPRECATED: This class will be removed soon.
   * @todo Remove this function.
   */
   function JustThisBitPlease( $type, $count=1 ) {
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'JustThisBitPlease' );
+    deprecated('iCalendar::JustThisBitPlease' );
     $answer = "";
     $intags = false;
     $start = "BEGIN:$type";
     $finish = "END:$type";
-    dbg_error_log( "iCalendar", ":JTBP: Looking for %d subsets of type %s", $count, $type );
+    dbg_error_log( 'iCalendar', ":JTBP: Looking for %d subsets of type %s", $count, $type );
     reset($this->lines);
     foreach( $this->lines AS $k => $v ) {
       if ( !$intags && $v == $start ) {
@@ -1118,17 +1130,17 @@ class iCalendar {
   * @var string The "SOMETHING" from the BEGIN:SOMETHING line we just met
   * @return arrayref An array of the things we found between (excluding) the BEGIN & END, some of which might be sub-arrays
   *
-  * @deprecated This function is deprecated and will be removed eventually.
+  * @DEPRECATED: This class will be removed soon.
   * @todo Remove this function.
   */
   function &ParseSomeLines( $type ) {
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'ParseSomeLines' );
+    deprecated('iCalendar::ParseSomeLines' );
     $props = array();
     $properties =& $props;
     while( isset($this->lines[$this->_current_parse_line]) ) {
       $i = $this->_current_parse_line++;
       $line =& $this->lines[$i];
-      dbg_error_log( "iCalendar", ":Parse:%s LINE %03d: >>>%s<<<", $type, $i, $line );
+      dbg_error_log( 'iCalendar', ":Parse:%s LINE %03d: >>>%s<<<", $type, $i, $line );
       if ( $this->parsing_vtimezone ) {
         $this->vtimezone .= $line."\n";
       }
@@ -1169,7 +1181,7 @@ class iCalendar {
               switch( $matches[1] ) {
                 case 'TZID': $properties['TZID'] = $matches[2];  break;
                 default:
-                  dbg_error_log( "icalendar", " FYI: Ignoring Resource '%s', Property '%s', Parameter '%s', Value '%s'", $type, $property, $matches[1], $matches[2] );
+                  dbg_error_log( 'iCalendar', " FYI: Ignoring Resource '%s', Property '%s', Parameter '%s', Value '%s'", $type, $property, $matches[1], $matches[2] );
               }
             }
           }
@@ -1189,11 +1201,11 @@ class iCalendar {
   *
   * @var string The RFC2445 iCalendar resource to be parsed
   *
-  * @deprecated This function is deprecated and will be removed eventually.
+  * @DEPRECATED: This class will be removed soon.
   * @todo Remove this function.
   */
   function BuildFromText( $icalendar ) {
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'BuildFromText' );
+    deprecated('iCalendar::BuildFromText' );
     /**
      * This unescapes the (CRLF + linear space) wrapping specified in RFC2445. According
      * to RFC2445 we should always end with CRLF but the CalDAV spec says that normalising
@@ -1226,11 +1238,11 @@ class iCalendar {
   * @param string $escaped The incoming string to be escaped.
   * @return string The string with RFC2445 content escaping removed.
   *
-  * @deprecated This function is deprecated and will be removed eventually.
+  * @DEPRECATED: This class will be removed soon.
   * @todo Remove this function.
   */
   function RFC2445ContentUnescape( $escaped ) {
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'RFC2445ContentUnescape' );
+    deprecated( 'RFC2445ContentUnescape' );
     $unescaped = str_replace( '\\n', "\n", $escaped);
     $unescaped = str_replace( '\\N', "\n", $unescaped);
     $unescaped = preg_replace( "/\\\\([,;:\"\\\\])/", '$1', $unescaped);
@@ -1243,21 +1255,21 @@ class iCalendar {
   * Do what must be done with time zones from on file.  Attempt to turn
   * them into something that PostgreSQL can understand...
   *
-  * @deprecated This function is deprecated and will be removed eventually.
+  * @DEPRECATED: This class will be removed soon.
   * @todo Remove this function.
   */
   function DealWithTimeZones() {
     global $c;
 
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'DealWithTimeZones' );
+    deprecated('iCalendar::DealWithTimeZones' );
     $tzid = $this->Get('TZID');
     if ( isset($c->save_time_zone_defs) && $c->save_time_zone_defs ) {
       $qry = new PgQuery( "SELECT tz_locn FROM time_zone WHERE tz_id = ?;", $tzid );
-      if ( $qry->Exec('iCalendar') && $qry->rows == 1 ) {
+      if ( $qry->Exec('iCalendar') && $qry->rows() == 1 ) {
         $row = $qry->Fetch();
         $this->tz_locn = $row->tz_locn;
       }
-      dbg_error_log( "icalendar", " TZCrap2: TZID '%s', DB Rows=%d, Location '%s'", $tzid, $qry->rows, $this->tz_locn );
+      dbg_error_log( 'iCalendar', " TZCrap2: TZID '%s', DB Rows=%d, Location '%s'", $tzid, $qry->rows(), $this->tz_locn );
     }
 
     if ( (!isset($this->tz_locn) || $this->tz_locn == '') && $tzid != '' ) {
@@ -1281,13 +1293,13 @@ class iCalendar {
       if ( preg_match( '#\S+/\S+#', $tzname) ) {
         $this->tz_locn = $tzname;
       }
-      dbg_error_log( "icalendar", " TZCrap3: TZID '%s', Location '%s', Perhaps: %s", $tzid, $this->tz_locn, $tzname );
+      dbg_error_log( 'iCalendar', " TZCrap3: TZID '%s', Location '%s', Perhaps: %s", $tzid, $this->tz_locn, $tzname );
     }
 
-    if ( $tzid != '' && isset($c->save_time_zone_defs) && $c->save_time_zone_defs && $qry->rows != 1 && isset($this->vtimezone) && $this->vtimezone != "" ) {
+    if ( $tzid != '' && isset($c->save_time_zone_defs) && $c->save_time_zone_defs && $qry->rows() != 1 && isset($this->vtimezone) && $this->vtimezone != "" ) {
       $qry2 = new PgQuery( "INSERT INTO time_zone (tz_id, tz_locn, tz_spec) VALUES( ?, ?, ? );",
                                    $tzid, $this->tz_locn, $this->vtimezone );
-      $qry2->Exec("iCalendar");
+      $qry2->Exec('iCalendar');
     }
 
     if ( (!isset($this->tz_locn) || $this->tz_locn == "") && isset($c->local_tzid) ) {
@@ -1298,11 +1310,13 @@ class iCalendar {
 
   /**
   * Get the value of a property in the first non-VTIMEZONE
+  * @DEPRECATED: This class will be removed soon.
   */
   function Get( $key ) {
+    deprecated('iCalendar::Get' );
     if ( strtoupper($key) == 'TZID' ) {
       // backward compatibility hack
-      dbg_error_log( "icalendar", " Get(TZID): TZID '%s', Location '%s'", (isset($this->tzid)?$this->tzid:"[not set]"), $this->tz_locn );
+      dbg_error_log( 'iCalendar', " Get(TZID): TZID '%s', Location '%s'", (isset($this->tzid)?$this->tzid:"[not set]"), $this->tz_locn );
       if ( isset($this->tzid) ) return $this->tzid;
       return $this->tz_locn;
     }
@@ -1317,8 +1331,10 @@ class iCalendar {
 
   /**
   * Set the value of a property
+  * @DEPRECATED: This class will be removed soon.
   */
   function Set( $key, $value ) {
+    deprecated('iCalendar::Set' );
     if ( $value == "" ) return;
     $key = strtoupper($key);
     $property = new iCalProp();
@@ -1332,6 +1348,7 @@ class iCalendar {
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Add a new property/value, regardless of whether it exists already
   *
   * @param string $key The property key
@@ -1339,6 +1356,7 @@ class iCalendar {
   * @param string $parameters Any parameters to set for the property, as an array of key/value pairs
   */
   function Add( $key, $value, $parameters = null ) {
+    deprecated('iCalendar::Add' );
     if ( $value == "" ) return;
     $key = strtoupper($key);
     $property = new iCalProp();
@@ -1354,6 +1372,7 @@ class iCalendar {
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Get all sub-components, or at least get those matching a type, or failling to match,
   * should the second parameter be set to false.
   *
@@ -1362,84 +1381,98 @@ class iCalendar {
   * @return array an array of the sub-components
   */
   function GetComponents( $type = null, $normal_match = true ) {
+    deprecated('iCalendar::GetComponents' );
     return $this->component->GetComponents($type,$normal_match);
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Clear all components, or the components matching a particular type
   * @param string $type The type of component - omit for all components
   */
   function ClearComponents( $type = null ) {
+    deprecated('iCalendar::ClearComponents' );
     $this->component->ClearComponents($type);
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Sets some or all sub-components of the component to the supplied new components
   *
   * @param array of iCalComponent $new_components The new components to replace the existing ones
   * @param string $type The type of components to be replaced.  Defaults to null, which means all components will be replaced.
   */
   function SetComponents( $new_component, $type = null ) {
+    deprecated('iCalendar::SetComponents' );
     $this->component->SetComponents( $new_component, $type );
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Adds a new subcomponent
   *
   * @param iCalComponent $new_component The new component to append to the set
   */
   function AddComponent( $new_component ) {
+    deprecated('iCalendar::AddComponent' );
     $this->component->AddComponent($new_component);
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Mask components, removing any that are not of the types in the list
   * @param array $keep An array of component types to be kept
   */
   function MaskComponents( $keep ) {
+    deprecated('iCalendar::MaskComponents' );
     $this->component->MaskComponents($keep);
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Returns a PostgreSQL Date Format string suitable for returning HTTP (RFC2068) dates
   * Preferred is "Sun, 06 Nov 1994 08:49:37 GMT" so we do that.
   */
-  function HttpDateFormat() {
+  static function HttpDateFormat() {
     return "'Dy, DD Mon IYYY HH24:MI:SS \"GMT\"'";
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Returns a PostgreSQL Date Format string suitable for returning iCal dates
   */
-  function SqlDateFormat() {
+  static function SqlDateFormat() {
     return "'YYYYMMDD\"T\"HH24MISS'";
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Returns a PostgreSQL Date Format string suitable for returning dates which
   * have been cast to UTC
   */
-  function SqlUTCFormat() {
+  static function SqlUTCFormat() {
     return "'YYYYMMDD\"T\"HH24MISS\"Z\"'";
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Returns a PostgreSQL Date Format string suitable for returning iCal durations
   *  - this doesn't work for negative intervals, but events should not have such!
   */
-  function SqlDurationFormat() {
+  static function SqlDurationFormat() {
     return "'\"PT\"HH24\"H\"MI\"M\"'";
   }
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Returns a suitably escaped RFC2445 content string.
   *
   * @param string $name The incoming name[;param] prefixing the string.
@@ -1449,7 +1482,7 @@ class iCalendar {
   * @todo Remove this function.
   */
   function RFC2445ContentEscape( $name, $value ) {
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'RFC2445ContentEscape' );
+    deprecated('iCalendar::RFC2445ContentEscape' );
     $property = preg_replace( '/[;].*$/', '', $name );
     switch( $property ) {
         /** Content escaping does not apply to these properties culled from RFC2445 */
@@ -1473,6 +1506,7 @@ class iCalendar {
   }
 
   /**
+  * @DEPRECATED: This class will be removed soon.
    * Return all sub-components of the given type, which are part of the
    * component we pass in as an array of lines.
    *
@@ -1483,11 +1517,12 @@ class iCalendar {
    * @return array The sub-component lines
    */
   function ExtractSubComponent( $component, $type, $count=9999 ) {
+    deprecated('iCalendar::ExtractSubComponent' );
     $answer = array();
     $intags = false;
     $start = "BEGIN:$type";
     $finish = "END:$type";
-    dbg_error_log( "iCalendar", ":ExtractSubComponent: Looking for %d subsets of type %s", $count, $type );
+    dbg_error_log( 'iCalendar', ":ExtractSubComponent: Looking for %d subsets of type %s", $count, $type );
     reset($component);
     foreach( $component AS $k => $v ) {
       if ( !$intags && $v == $start ) {
@@ -1507,6 +1542,7 @@ class iCalendar {
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
    * Extract a particular property from the provided component.  In doing so we
    * assume that the content was unescaped when iCalComponent::ParseFrom()
    * called iCalComponent::UnwrapComponent().
@@ -1517,13 +1553,14 @@ class iCalendar {
    * @return array An array of iCalProperty objects
    */
   function ExtractProperty( $component, $type, $count=9999 ) {
+    deprecated('iCalendar::ExtractProperty' );
     $answer = array();
-    dbg_error_log( "iCalendar", ":ExtractProperty: Looking for %d properties of type %s", $count, $type );
+    dbg_error_log( 'iCalendar', ":ExtractProperty: Looking for %d properties of type %s", $count, $type );
     reset($component);
     foreach( $component AS $k => $v ) {
       if ( preg_match( "/$type"."[;:]/i", $v ) ) {
         $answer[] = new iCalProp($v);
-        dbg_error_log( "iCalendar", ":ExtractProperty: Found property %s", $type );
+        dbg_error_log( 'iCalendar', ":ExtractProperty: Found property %s", $type );
         if ( --$count < 1 ) return $answer;
       }
     }
@@ -1532,6 +1569,7 @@ class iCalendar {
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
    * Applies the filter conditions, possibly recursively, to the value which will be either
    * a single property, or an array of lines of the component under test.
    *
@@ -1547,16 +1585,17 @@ class iCalendar {
    * @return boolean Whether the filter passed / failed.
    */
   function ApplyFilter( $filter, $value ) {
+    deprecated('iCalendar::ApplyFilter' );
     foreach( $filter AS $k => $v ) {
       $tag = $v->GetTag();
       $value_type = gettype($value);
       $value_defined = (isset($value) && $value_type == 'string') || ($value_type == 'array' && count($value) > 0 );
       if ( $tag == 'urn:ietf:params:xml:ns:caldav:is-not-defined' && $value_defined ) {
-        dbg_error_log( "iCalendar", ":ApplyFilter: Value is set ('%s'), want unset, for filter %s", count($value), $tag );
+        dbg_error_log( 'iCalendar', ":ApplyFilter: Value is set ('%s'), want unset, for filter %s", count($value), $tag );
         return false;
       }
       elseif ( $tag == 'urn:ietf:params:xml:ns:caldav:is-defined' && !$value_defined ) {
-        dbg_error_log( "iCalendar", ":ApplyFilter: Want value, but it is not set for filter %s", $tag );
+        dbg_error_log( 'iCalendar', ":ApplyFilter: Want value, but it is not set for filter %s", $tag );
         return false;
       }
       else {
@@ -1577,7 +1616,7 @@ class iCalendar {
                 }
               }
               else {
-                dbg_error_log( "iCalendar", ":ApplyFilter: TEXT-MATCH will only work on strings or arrays of iCalProp.  %s unsupported", gettype($value) );
+                dbg_error_log( 'iCalendar', ":ApplyFilter: TEXT-MATCH will only work on strings or arrays of iCalProp.  %s unsupported", gettype($value) );
                 return true;  // We return _true_ in this case, so the client sees the item
               }
             }
@@ -1586,7 +1625,7 @@ class iCalendar {
             }
             $negate = $v->GetAttribute("negate-condition");
             if ( isset($negate) && strtolower($negate) == "yes" && $match ) {
-              dbg_error_log( "iCalendar", ":ApplyFilter: TEXT-MATCH of %s'%s' against '%s'", (isset($negate) && strtolower($negate) == "yes"?'!':''), $search, $value );
+              dbg_error_log( 'iCalendar', ":ApplyFilter: TEXT-MATCH of %s'%s' against '%s'", (isset($negate) && strtolower($negate) == "yes"?'!':''), $search, $value );
               return false;
             }
             break;
@@ -1612,6 +1651,7 @@ class iCalendar {
   }
 
   /**
+  * @DEPRECATED: This class will be removed soon.
    * Test a PROP-FILTER or COMP-FILTER and return a true/false
    * COMP-FILTER (is-defined | is-not-defined | (time-range?, prop-filter*, comp-filter*))
    * PROP-FILTER (is-defined | is-not-defined | ((time-range | text-match)?, param-filter*))
@@ -1621,6 +1661,7 @@ class iCalendar {
    * @return boolean Whether or not this iCalendar passes the test
    */
   function TestFilter( $filters ) {
+    deprecated('iCalendar::TestFilter' );
 
     foreach( $filters AS $k => $v ) {
       $tag = $v->GetTag();
@@ -1639,13 +1680,13 @@ class iCalendar {
   }
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Returns the header we always use at the start of our iCalendar resources
   *
-  * @deprecated This function is deprecated and will be removed eventually.
   * @todo Remove this function.
   */
-  function iCalHeader() {
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'iCalHeader' );
+  static function iCalHeader() {
+    deprecated('iCalendar::iCalHeader' );
     return <<<EOTXT
 BEGIN:VCALENDAR\r
 PRODID:-//davical.org//NONSGML AWL Calendar//EN\r
@@ -1657,18 +1698,19 @@ EOTXT;
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Returns the footer we always use at the finish of our iCalendar resources
   *
-  * @deprecated This function is deprecated and will be removed eventually.
   * @todo Remove this function.
   */
-  function iCalFooter() {
-    dbg_error_log( "LOG", " iCalendar: Call to deprecated method '%s'", 'iCalFooter' );
+  static function iCalFooter() {
+    deprecated('iCalendar::iCalFooter' );
     return "END:VCALENDAR\r\n";
   }
 
 
   /**
+  * @DEPRECATED: This class will be removed soon.
   * Render the iCalendar object as a text string which is a single VEVENT (or other)
   *
   * @param boolean $as_calendar Whether or not to wrap the event in a VCALENDAR
@@ -1676,6 +1718,7 @@ EOTXT;
   * @param array $restrict_properties The names of the properties we want in our rendered result.
   */
   function Render( $as_calendar = true, $type = null, $restrict_properties = null ) {
+    deprecated('iCalendar::Render' );
     if ( $as_calendar ) {
       return $this->component->Render();
     }
